#!/usr/bin/env bash

# vim: tabstop=4 shiftwidth=4 softtabstop=4
#
#  Copyright (c) 2011 Openstack, LLC.
#  All Rights Reserved.
#
#     Licensed under the Apache License, Version 2.0 (the "License"); you may
#     not use this file except in compliance with the License. You may obtain
#     a copy of the License at
#
#          http://www.apache.org/licenses/LICENSE-2.0
#
#     Unless required by applicable law or agreed to in writing, software
#     distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#     WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#     License for the specific language governing permissions and limitations
#     under the License.
#

prefix="/usr"
exec_prefix="${prefix}"
sbindir="/usr/bin"
datadir="${prefix}/share/nova-agent"
dataversdir="${prefix}/share/nova-agent/1.39.1"
reallibdir="${prefix}/share/nova-agent/1.39.1/lib"
version="1.39.1"

umask 077

my_dir=`dirname $0`

add_to_startup() {
    if [ -f /sbin/insserv -o -f /usr/sbin/update-rc.d ] ; then
        # Debian style
        if [ -f /sbin/insserv ]; then
            # Debian >= 6.0
            /sbin/insserv nova-agent > /dev/null 2>&1
        else
            /usr/sbin/update-rc.d nova-agent start 20 2 3 4 5 . stop 20 0 1 6 . > /dev/null 2>&1
        fi
    elif [ -f /sbin/chkconfig ] ; then
        # Red Hat style
        /sbin/chkconfig nova-agent on > /dev/null 2>&1
    elif [ -f /sbin/rc-update ] ; then
        # Gentoo style
        if [ -f /etc/init.d/nova-agent.gentoo.init ]; then
            /sbin/rc-update del nova-agent.gentoo.init > /dev/null 2>&1
            rm /etc/init.d/nova-agent.gentoo.init
        fi

        /sbin/rc-update add nova-agent default > /dev/null 2>&1
    elif [ -f /etc/arch-release ] ; then
        if [[ "$(basename $(readlink /sbin/init))" == systemd ]]; then
            /usr/bin/systemctl enable -f nova-agent.service
        else
            # Arch is a hybrid BSD and SysV style init system. We just need to
            # install a SysV style script into /etc/rc.d, but then update
            # /etc/rc.conf to enable it
            grep -q '^DAEMONS=.*nova-agent' /etc/rc.conf
            if [ $? -ne 0 ] ; then
                sed -i.backup 's/^DAEMONS=(\(.*\))/DAEMONS=(nova-agent \1)/' /etc/rc.conf
            fi
        fi
    elif [ `uname -s` == 'FreeBSD' ] ; then
        # FreeBSD needs nova_agent_enable="YES" added to rc.conf
        egrep -q '^nova_agent_enable=.*YES' /etc/rc.conf
        if [ $? -ne 0 ] ; then
            sed -i.backup 's/^.*nova_agent.*//' /etc/rc.conf
            cat >> /etc/rc.conf << EOF
# DO NOT DISABLE nova_agent or you will lose some functionality
nova_agent_enable="YES"
EOF
        fi
    elif [ -f /etc/rc.local ] ; then
        grep -q nova-agent /etc/rc.local
        if [ $? -ne 0 ] ; then
            if [ -d /etc/rc.d ]; then
                echo "/etc/rc.d/nova-agent start" >> /etc/rc.local
            elif [ -d /etc/init.d ]; then
                echo "/etc/init.d/nova-agent start" >> /etc/rc.local
            else
                echo "/etc/nova-agent.init start" >> /etc/rc.local
            fi
        fi
    fi
}

if [ -d $dataversdir ] ; then
    # This version is already installed
    echo "Version ${version} is already installed"
    exit 1
fi

if [ ! -d $datadir ] ; then
    mkdir $datadir
fi

# Copy our version subdirectory to the target
cp -R ${my_dir}/${dataversdir} ${dataversdir}

# Creating Environment Variable file for SYSTEMD service script
## condition checking if /sbin/init is symlink to systemd
if [[ -L /sbin/init && "$(basename $(readlink /sbin/init))" == systemd ]]; then
  if [ -f /etc/arch-release ] ; then
    NOVA_PYTHONPATH=`ls -l ${reallibdir} | grep '^d' | awk '{print \$NF}'`
    NOVA_PYTHONPATH=`echo ${NOVA_PYTHONPATH} | grep 'python[0-9]\.[0-9]'`
    NOVA_PYTHONPATH="${reallibdir}/${NOVA_PYTHONPATH}"
    NOVA_PYTHONPATH="${NOVA_PYTHONPATH}:${NOVA_PYTHONPATH}/site-packages"

    PYTHONPATH="$(python2 -c 'import sys; print ":".join(sys.path)')"
    PYTHONPATH="${NOVA_PYTHONPATH}:${PYTHONPATH}"

    cat >> /etc/nova-agent.env <<-EOF
PYTHONPATH=$PYTHONPATH
EOF
  else
    touch /etc/nova-agent.env
  fi
fi

# Install the init file symlink
if [ -h /sbin/runscript ]; then
    # Gentoo Linux OpenRC style
    rm -f /etc/init.d/nova-agent
    ln -s ${dataversdir}/etc/gentoo/nova-agent /etc/init.d/nova-agent
elif [ -d /etc/init.d ]; then
    # Generic Linux
    rm -f /etc/init.d/nova-agent
    ln -s ${dataversdir}/etc/generic/nova-agent /etc/init.d/nova-agent
elif [ -f /etc/arch-release ] ; then
    if [[ "$(basename $(readlink /sbin/init))" == systemd ]]; then
        ln -sf ${dataversdir}/etc/systemd/system/nova-agent.service /etc/systemd/system/nova-agent.service
    else
        # Arch Linux
        rm -f /etc/rc.d/nova-agent
        ln -s ${dataversdir}/etc/generic/nova-agent /etc/rc.d/nova-agent
    fi
elif [ -d /etc/rc.d ]; then
    # FreeBSD
    rm -f /etc/rc.d/nova-agent
    ln -s ${dataversdir}/etc/freebsd/nova-agent /etc/rc.d/nova-agent
else
    # Unknown, just try something and hope it works
    rm -f /etc/nova-agent.init
    ln -s ${dataversdir}/etc/generic/nova-agent /etc/nova-agent.init
fi

# Make sure we actually run the init script on boot
add_to_startup

# Install the new nova-agent symlink
rm -f ${sbindir}/nova-agent
ln -s ${dataversdir}/sbin/nova-agent ${sbindir}/nova-agent 

# Install the new nova-agent config file symlink
rm -f ${datadir}/nova-agent.py
ln -s ${dataversdir}/nova-agent.py ${datadir}/nova-agent.py

# Remove the old agent if it exists
if [ -f /usr/sbin/agent-smith ] ; then
    rm -f /usr/sbin/agent-smith
    rm -rf /usr/share/agent-smith
    rm -f /var/log/agent-smith.log
fi

exit 0
